"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TRenderEngine = void 0;

var _collapse = require("./flow/collapse");

var _hoist = require("./flow/hoist");

var _translate = require("./flow/translate");

var _htmlparser = require("htmlparser2");

var _omit = _interopRequireDefault(require("ramda/src/omit"));

var _cssProcessor = require("@native-html/css-processor");

var _TStylesMerger = require("./styles/TStylesMerger");

var _defaults = require("./styles/defaults");

var _TStyles = require("./styles/TStyles");

var _HTMLModelRegistry = _interopRequireDefault(require("./model/HTMLModelRegistry"));

var _alterDOMNodes = _interopRequireDefault(require("./dom/alterDOMNodes"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function createStylesConfig(options) {
  var _options$stylesConfig, _options$stylesConfig2;

  const enableUserAgentStyles = typeof (options === null || options === void 0 ? void 0 : (_options$stylesConfig = options.stylesConfig) === null || _options$stylesConfig === void 0 ? void 0 : _options$stylesConfig.enableUserAgentStyles) === 'boolean' ? options.stylesConfig.enableUserAgentStyles : _defaults.defaultStylesConfig.enableUserAgentStyles;
  const baseStyle = { ...(enableUserAgentStyles ? _defaults.defaultStylesConfig.baseStyle : (0, _omit.default)(['fontSize'], _defaults.defaultStylesConfig.baseStyle)),
    ...(options === null || options === void 0 ? void 0 : (_options$stylesConfig2 = options.stylesConfig) === null || _options$stylesConfig2 === void 0 ? void 0 : _options$stylesConfig2.baseStyle)
  };
  return { ..._defaults.defaultStylesConfig,
    ...(options === null || options === void 0 ? void 0 : options.stylesConfig),
    baseStyle
  };
}

class TRenderEngine {
  constructor(options) {
    var _options$cssProcessor, _stylesConfig$baseSty;

    _defineProperty(this, "htmlParserOptions", void 0);

    _defineProperty(this, "dataFlowParams", void 0);

    _defineProperty(this, "alterDOMParams", void 0);

    const stylesConfig = createStylesConfig(options);
    const modelRegistry = new _HTMLModelRegistry.default(options === null || options === void 0 ? void 0 : options.customizeHTMLModels);
    const userSelectedFontSize = (options === null || options === void 0 ? void 0 : (_options$cssProcessor = options.cssProcessorConfig) === null || _options$cssProcessor === void 0 ? void 0 : _options$cssProcessor.rootFontSize) || ((_stylesConfig$baseSty = stylesConfig.baseStyle) === null || _stylesConfig$baseSty === void 0 ? void 0 : _stylesConfig$baseSty.fontSize); // TODO log a warning when type is string

    const stylesMerger = new _TStylesMerger.TStylesMerger(stylesConfig, modelRegistry, { ..._cssProcessor.defaultCSSProcessorConfig,
      ...(options === null || options === void 0 ? void 0 : options.cssProcessorConfig),
      rootFontSize: typeof userSelectedFontSize === 'number' ? userSelectedFontSize : 14
    });
    this.alterDOMParams = options === null || options === void 0 ? void 0 : options.alterDOMParams;
    this.htmlParserOptions = {
      decodeEntities: true,
      ...(options === null || options === void 0 ? void 0 : options.htmlParserOptions)
    };
    this.dataFlowParams = {
      stylesMerger,
      modelRegistry,
      baseStyles: new _TStyles.TStyles(stylesMerger.compileStyleDeclaration(stylesConfig.baseStyle)),
      removeLineBreaksAroundEastAsianDiscardSet: (options === null || options === void 0 ? void 0 : options.removeLineBreaksAroundEastAsianDiscardSet) || false
    };
  }

  buildTTree(html) {
    var _this$alterDOMParams, _this$alterDOMParams2, _this$alterDOMParams3, _this$alterDOMParams4;

    let documentTree = (0, _htmlparser.parseDOM)(html, this.htmlParserOptions);

    if (((_this$alterDOMParams = this.alterDOMParams) === null || _this$alterDOMParams === void 0 ? void 0 : _this$alterDOMParams.ignoreDOMNode) || ((_this$alterDOMParams2 = this.alterDOMParams) === null || _this$alterDOMParams2 === void 0 ? void 0 : _this$alterDOMParams2.alterDOMChildren) || ((_this$alterDOMParams3 = this.alterDOMParams) === null || _this$alterDOMParams3 === void 0 ? void 0 : _this$alterDOMParams3.alterDOMData) || ((_this$alterDOMParams4 = this.alterDOMParams) === null || _this$alterDOMParams4 === void 0 ? void 0 : _this$alterDOMParams4.alterDOMElement)) {
      documentTree = (0, _alterDOMNodes.default)(documentTree, this.alterDOMParams);
    }

    const tdoc = (0, _translate.translateDocument)(documentTree, this.dataFlowParams);
    return (0, _collapse.collapse)((0, _hoist.hoist)(tdoc), this.dataFlowParams);
  }

}

exports.TRenderEngine = TRenderEngine;
//# sourceMappingURL=TRenderEngine.js.map