"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isSerializableText = isSerializableText;
exports.isSerializableElement = isSerializableElement;
exports.toSerializableChildren = toSerializableChildren;
exports.toSerializableNode = toSerializableNode;

var _utils = require("./utils");

function isSerializableText(node) {
  return node && node.type === 'text';
}

function isSerializableElement(node) {
  return node && node.type === 'element';
}

function toSerializableChildren(children) {
  const serializableNodes = [];

  for (const child of children) {
    const serialized = toSerializableNode(child);

    if (serialized) {
      serializableNodes.push(serialized);
    }
  }

  return serializableNodes;
}

function serializeTextNode({
  data
}) {
  return {
    data,
    type: 'text'
  };
}

function toSerializableElement({
  attribs,
  tagName,
  children
}) {
  return {
    attribs,
    tagName,
    type: 'element',
    children: toSerializableChildren(children)
  };
}

function toSerializableNode(node) {
  if ((0, _utils.isTextNode)(node)) {
    return serializeTextNode(node);
  }

  if ((0, _utils.isElementNode)(node)) {
    return toSerializableElement(node);
  }

  return null;
}
//# sourceMappingURL=to-serializable.js.map