"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.collapse = collapse;

var _TBlock = require("../tree/TBlock");

var _TPhrasing = require("../tree/TPhrasing");

var _TText = require("../tree/TText");

var _textTransforms = require("./text-transforms");

var _compose = _interopRequireDefault(require("ramda/src/compose"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const collapseWhiteSpaces = (0, _compose.default)(_textTransforms.normalizeWhitespaces, _textTransforms.replaceSegmentBreaks, _textTransforms.normalizeZeroWidthWhitespaces, _textTransforms.removeConsecutiveSegmentBreaks, _textTransforms.removeCollapsibleAroundSegmentBreak);
const collapseWhiteSpacesWithEastAsianCharset = (0, _compose.default)(_textTransforms.normalizeWhitespaces, _textTransforms.replaceSegmentBreaks, _textTransforms.removeLineBreaksAroundEastAsianDiscardSet, _textTransforms.normalizeZeroWidthWhitespaces, _textTransforms.removeConsecutiveSegmentBreaks, _textTransforms.removeCollapsibleAroundSegmentBreak);

function collapseText(node, params) {
  if (node.hasWhiteSpaceCollapsingEnabled) {
    if (params.removeLineBreaksAroundEastAsianDiscardSet) {
      node.data = collapseWhiteSpacesWithEastAsianCharset(node.data);
    } else {
      node.data = collapseWhiteSpaces(node.data);
    }
  }

  return node;
}

function collapseBlock(node, params) {
  const newChildren = [];

  for (const i in node.children) {
    const child = collapseNode(node.children[i], params);

    if (!child.isWhitespace()) {
      child.trimLeft();
      child.trimRight();

      if (!child.isEmpty()) {
        newChildren.push(child);
      }
    }
  }

  node.bindChildren(newChildren);
  return node;
}

function collapsePhrasing(node, params) {
  let collapsedChildren = [];
  let trimmedChildren = [];

  for (const i in node.children) {
    const child = collapseNode(node.children[i], params);

    if (child instanceof _TText.TText) {
      if (!child.isEmpty()) {
        collapsedChildren.push(child);
      }
    } else if (child instanceof _TPhrasing.TPhrasing) {
      if (!child.isEmpty()) {
        collapsedChildren.push(child);
      }
    }
  }

  collapsedChildren.forEach((childI, i) => {
    const k = i + 1;

    if (k <= node.children.length - 1) {
      const childK = node.children[k];

      if (childI.isCollapsibleRight() && childK.isCollapsibleLeft()) {
        childI.trimRight();

        if (!childI.isEmpty()) {
          trimmedChildren.push(childI);
        }
      } else {
        trimmedChildren.push(childI);
      }
    } else {
      trimmedChildren.push(childI);
    }
  });
  node.bindChildren(trimmedChildren);
  return node;
}

function collapseNode(node, params) {
  if (node instanceof _TText.TText) {
    return collapseText(node, params);
  }

  if (node instanceof _TBlock.TBlock) {
    return collapseBlock(node, params);
  }

  if (node instanceof _TPhrasing.TPhrasing) {
    return collapsePhrasing(node, params);
  }

  return node;
}

function collapse(tree, params) {
  const root = collapseNode(tree, params);
  root.trimLeft();
  root.trimRight();
  return root;
}
//# sourceMappingURL=collapse.js.map