"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hoist = hoist;

var _TBlock = require("../tree/TBlock");

var _TPhrasing = require("../tree/TPhrasing");

var _TText = require("../tree/TText");

/**
 * Wrap text nodes around TPhrasing nodes.
 * @param tnode The parent node of all elements to group.
 */
function groupText(tnode) {
  let newChildren = [];
  const wrappernode = new _TPhrasing.TPhrasing({
    parentStyles: tnode.styles,
    contentModel: null,
    elementModel: null,
    stylesMerger: tnode.stylesMerger,
    domNode: null,
    nodeIndex: 0,
    parent: null
  });
  let wrapper = wrappernode.newEmpty();

  for (const child of tnode.children) {
    if (child instanceof _TText.TText || child instanceof _TPhrasing.TPhrasing) {
      wrapper.children.push(child);
    } else {
      if (wrapper.children.length) {
        newChildren.push(wrapper);
        wrapper = wrappernode.newEmpty();
      }

      newChildren.push(child);
    }
  }

  if (wrapper.children.length) {
    newChildren.push(wrapper);
  }

  tnode.bindChildren(newChildren);
  return tnode;
}

function hoistNode(tnode) {
  tnode.bindChildren(tnode.children.map(hoistNode));

  if (tnode instanceof _TPhrasing.TPhrasing) {
    for (const cnode of tnode.children) {
      if (cnode instanceof _TBlock.TBlock) {
        const initParams = tnode.cloneInitParams({
          parentStyles: cnode.parentStyles,
          styles: cnode.styles
        });
        const newNode = new _TBlock.TBlock(initParams);
        newNode.bindChildren(tnode.children);
        const output = groupText(newNode);
        return output;
      }
    }
  } else if (tnode instanceof _TBlock.TBlock) {
    if (tnode.children.length > 0) {
      const output = groupText(tnode);
      return output;
    }
  }

  return tnode;
}

function hoist(tree) {
  return hoistNode(tree);
}
//# sourceMappingURL=hoist.js.map