"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TDocument = void 0;

var _TEmpty = require("./TEmpty");

var _TBlock = require("./TBlock");

var _toSerializable = require("../dom/to-serializable");

var _HTMLElementModel = _interopRequireDefault(require("../model/HTMLElementModel"));

var _defaultHTMLElementModels = _interopRequireDefault(require("../model/defaultHTMLElementModels"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const defaultContextBase = Object.freeze({
  baseHref: 'about:blank',
  baseTarget: '_self',
  charset: 'utf-8',
  title: '',
  lang: 'en',
  dir: 'ltr',
  links: [],
  meta: []
});

function getDefaultDocumentContext() {
  return Object.assign({}, defaultContextBase, {
    links: [],
    meta: []
  });
}

function extractContextFromHead(head, lang, dir) {
  const context = getDefaultDocumentContext();

  if (lang) {
    context.lang = lang;
  }

  if (dir && dir === 'rtl') {
    context.dir = 'rtl';
  }

  const domNode = head.domNode;
  const children = domNode.children;
  children.filter(_toSerializable.isSerializableElement).forEach(child => {
    if (child.tagName === 'meta') {
      if (child.attribs.name) {
        context.meta.push(child.attribs);
      } else if (child.attribs.charset) {
        context.charset = child.attribs.charset.toLowerCase();
      }
    } else if (child.tagName === 'link') {
      context.links.push(child.attribs);
    } else if (child.tagName === 'title') {
      for (const titleChild of child.children.filter(_toSerializable.isSerializableText)) {
        context.title = titleChild.data.trim();
        break;
      }
    } else if (child.tagName === 'base') {
      context.baseHref = child.attribs.href || context.baseHref;
      context.baseTarget = child.attribs.target || context.baseTarget;
    }
  });
  return context;
}

const htmlModel = _HTMLElementModel.default.fromNativeModel({
  tagName: 'html',
  category: 'grouping'
});

class TDocument extends _TBlock.TBlock {
  constructor({
    attributes,
    stylesMerger,
    styles,
    domNode
  }) {
    super({
      tagName: 'html',
      styles,
      attributes,
      contentModel: htmlModel.contentModel,
      elementModel: htmlModel,
      parentStyles: null,
      stylesMerger,
      domNode,
      nodeIndex: 0,
      parent: null
    }); // @ts-ignore

    _defineProperty(this, "context", void 0);

    _defineProperty(this, "displayName", 'TDocument');

    this.type = 'document';
    this.context = defaultContextBase;
  }
  /**
   * Iterate over children and extract meta-information into context field.
   * Replace children with a single-element array containing the body.
   */


  parseChildren() {
    let head;
    let body;

    for (const child of this.children) {
      if (head && body) {
        break;
      }

      if (child.tagName === 'head') {
        head = child;
      } else if (child.tagName === 'body') {
        body = child;
      }
    }

    this.bindChildren([body || new _TBlock.TBlock({
      tagName: 'body',
      contentModel: _defaultHTMLElementModels.default.body.contentModel,
      elementModel: _defaultHTMLElementModels.default.body,
      stylesMerger: this.stylesMerger,
      parentStyles: this.styles,
      domNode: null,
      nodeIndex: 0,
      parent: this
    })]); //@ts-ignore

    this.context = Object.freeze(extractContextFromHead(head || new _TEmpty.TEmpty({
      tagName: 'head',
      isUnregistered: false,
      stylesMerger: this.stylesMerger,
      parentStyles: null,
      contentModel: _defaultHTMLElementModels.default.head.contentModel,
      elementModel: _defaultHTMLElementModels.default.head,
      domNode: {
        type: 'element',
        attribs: {},
        children: [],
        tagName: 'head'
      },
      nodeIndex: 0,
      parent: this
    }), this.attributes.lang, this.attributes.dir));
  }

}

exports.TDocument = TDocument;
//# sourceMappingURL=TDocument.js.map