"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TNode = void 0;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class TNode {
  constructor(init, type) {
    _defineProperty(this, "type", void 0);

    _defineProperty(this, "styles", void 0);

    _defineProperty(this, "attributes", void 0);

    _defineProperty(this, "contentModel", void 0);

    _defineProperty(this, "elementModel", void 0);

    _defineProperty(this, "children", void 0);

    _defineProperty(this, "domNode", void 0);

    _defineProperty(this, "tagName", void 0);

    _defineProperty(this, "className", void 0);

    _defineProperty(this, "id", void 0);

    _defineProperty(this, "parentStyles", void 0);

    _defineProperty(this, "parent", void 0);

    _defineProperty(this, "hasWhiteSpaceCollapsingEnabled", void 0);

    _defineProperty(this, "stylesMerger", void 0);

    _defineProperty(this, "nodeIndex", void 0);

    _defineProperty(this, "displayName", void 0);

    this.type = type;
    this.attributes = init.attributes || {};
    this.contentModel = init.contentModel;
    this.tagName = init.tagName || null;
    this.elementModel = init.elementModel;
    this.domNode = init.domNode || null;
    this.id = this.attributes.id || null;
    this.className = this.attributes.class || null;
    this.nodeIndex = init.nodeIndex;
    Object.defineProperty(this, 'stylesMerger', {
      enumerable: false,
      value: init.stylesMerger
    });
    Object.defineProperty(this, 'parent', {
      enumerable: false,
      value: init.parent
    });
    this.styles = init.styles || init.stylesMerger.buildStyles(this.attributes.style, init.parentStyles, this);
    this.parentStyles = init.parentStyles;
    this.children = [];
    this.hasWhiteSpaceCollapsingEnabled = typeof this.styles.webTextFlow.whiteSpace === 'string' ? this.styles.webTextFlow.whiteSpace === 'normal' : true;
    delete this.attributes.style;
    delete this.attributes.class;
    delete this.attributes.id;
  }

  bindChildren(children) {
    // @ts-expect-error
    this.children = children;
  }

  cloneInitParams(partial) {
    return { ...this,
      stylesMerger: this.stylesMerger,
      ...partial
    };
  }

  isCollapsibleLeft() {
    if (this.children.length) {
      return this.hasWhiteSpaceCollapsingEnabled && this.children[0].isCollapsibleLeft();
    }

    return false;
  }

  isCollapsibleRight() {
    if (this.children.length) {
      return this.hasWhiteSpaceCollapsingEnabled && this.children[this.children.length - 1].isCollapsibleRight();
    }

    return false;
  }

  isWhitespace() {
    return false;
  }

  isEmpty() {
    return false;
  }

  trimLeft() {
    if (this.children.length) {
      const firstChild = this.children[0];
      firstChild.trimLeft();

      if (firstChild.isEmpty()) {
        this.children.splice(0, 1);
      }
    }
  }

  trimRight() {
    if (this.children.length) {
      const lastChild = this.children[this.children.length - 1];
      lastChild.trimRight();

      if (lastChild.isEmpty()) {
        this.children.splice(-1, 1);
      }
    }
  }

}

exports.TNode = TNode;
//# sourceMappingURL=TNode.js.map